@@ -11,7 +11,7 @@ use strict;
 use warnings;
 
 our $PACK    = __PACKAGE__;
-our $VERSION = '0.09';
+our $VERSION = '0.10';
 ### @ISA     = qw( );
 
 # WARN:  Don't use POE or the POE::Kernel classes here. This is a
@@ -104,6 +104,12 @@ sub hasRouteBack { $_[0]->{header} and $_[0]->{header}->hasRouteBack()   }
 *getRouteTo   = \&hasRouteTo;
 *getRouteBack = \&hasRouteBack;
 
+sub nextRouteType     { $_[0]->{header}->nextRouteType()     if $_[0]->{header}}
+sub nextRouteIsRemote { $_[0]->{header}->nextRouteIsRemote() if $_[0]->{header}}
+sub nextRouteIsLocal  { $_[0]->{header}->nextRouteIsLocal()  if $_[0]->{header}}
+sub nextRouteIsPost   { $_[0]->{header}->nextRouteIsPost()   if $_[0]->{header}}
+sub nextRouteIsCall   { $_[0]->{header}->nextRouteIsCall()   if $_[0]->{header}}
+
 sub id     { $_[0]->{header}->get('id')  }    # unique message ID
 sub r2id   { $_[0]->{header}->get('r2id')}    # orig. unique message ID
 sub header { $_[0]->{header} }                                 # get only
@@ -209,6 +215,8 @@ sub _postOrCall                                      # For immediate routing
 	$argRef = [ @args ];
     }
 
+    # FIX: handle 'call' return here as done in sub '_autoRouting'
+
     if (! POE::Kernel->$mode( $session, $event, $argRef, [ $self ] ) ) {
 
 	my $msg = "Error(1): POE_Kernel->$mode( $session, $event, [ $argRef ], [ $self ] )";
@@ -232,9 +240,18 @@ sub route                              # forward, if possible, or reply
 {   my($self,@newArgs) = @_;
 
     # routeTo() until empty, then routeBack()
-    # returns:   1=routed,  0=nowhere to route
-    #
-    return $self->routeTo( @newArgs ) || $self->routeBack( @newArgs );
+    # 'post' returns:   1=routed,  0=nowhere to route
+    # 'call' returns:   whatever 'called' method returns
+
+    my(@list) = ();
+
+    if ( $self->hasRouteTo() ) {
+	(@list) = $self->routeTo( @newArgs );
+    } elsif ( $self->hasRouteBack() ) {
+	(@list) = $self->routeBack( @newArgs );
+    }
+
+    return( @list );
 }
 
 #---------------------------------------------
@@ -246,8 +263,8 @@ sub route                              # forward, if possible, or reply
 sub routeTo                            # activate "auto-forward", if any
 {   my($self,@newArgs) = @_;
 
-  # return undef if $self->stat();
-    return $self->_autoRouting( "delRouteTo", @newArgs );
+    my(@list) = $self->_autoRouting( "delRouteTo", @newArgs );
+    return( @list );
 }
 
 #---------------------------------------------
@@ -259,8 +276,8 @@ sub routeTo                            # activate "auto-forward", if any
 sub routeBack                          # activate "auto-reply", if any
 {   my($self,@newArgs) = @_;
 
-  # return undef if $self->stat();
-    return $self->_autoRouting( "delRouteBack", @newArgs );
+    my(@list) = $self->_autoRouting( "delRouteBack", @newArgs );
+    return( @list );
 }
 
 #---------------------------------------------
@@ -388,7 +405,9 @@ sub _autoRouting                       # activate "auto-routing," if any
 	(@args) = ( [ @origArgs ], [ $self, @newArgs ] );
     }
 
-    if (! POE::Kernel->$mode( $session, $event, @args ) ) {
+    my(@list) = POE::Kernel->$mode( $session, $event, @args );
+
+    if (! @list ) {
 
 	if ($!) {
 	    my $err = "Error(2): POE_Kernel->$mode( $session, $event, @args )";
@@ -399,6 +418,12 @@ sub _autoRouting                       # activate "auto-routing," if any
     }
 
     ## warn "DEBUG: Route: POE_Kernel->$mode( $session, $event, @args )\n";
+
+    # Here we return the same result as the POE::Kernel
+    # .  for 'post' return a boolean indicating success
+    # .  for 'call' return what the called method returns
+    #
+    return( @list )  if ($mode eq 'call');
     return 1;
 }
 
@@ -553,68 +578,117 @@ This document describes version 0.05, released December, 2005.
 
 =head1 SYNOPSIS
 
+ use POE;
  use POE::Event::Message;
 
  $MsgClass = "POE::Event::Message";
 
+
  # Creating Messages
 
- $mesg = $MsgClass->new();
- $body = <whatever>;              # almost anything 
+ $body = <whatever>;              #  body can be almost anything 
+ $mesg = $MsgClass->new( undef, $body );
 
- $mesg = $MsgClass->package( $body );
+ $mesg = $MsgClass->package( $body );   # alternative to 'new()' 
 
- # Accessing Messages
-
- $mesg->body( $body );            # add body
- $body = $mesg->body();           # get body
- $head = $mesg->header();         # get header
-
- print $mesg->dump();             # useful when debugging
- print $head->dump();             # useful when debugging
 
  # Creating Responses to Messages
-
- $resp = $MsgClass->new( $mesg->header()        );
- $resp = $MsgClass->new( $mesg->header(), $body );
+ # Note that every message has a unique identifier. For
+ # replies, the original 'message identifier' is retained
+ # as an 'in response to' identifier.
 
  $resp = $MsgClass->new( $mesg        );
  $resp = $MsgClass->new( $mesg, $body );
 
- # Routing Messages and "Auto Replies" (Within POE environment)
-
- $mesg->header->setMode( "call" );    # default mode: "post"
+ $resp = $MsgClass->new( $mesg->header()        );
+ $resp = $MsgClass->new( $mesg->header(), $body );
 
 
- # FIX: replace this section
+ # Accessing Message Components
 
- my(@routeback) = ( $mode => $session, $event, @routebackArgs );
- $mesg->header->addRouteBack( @routeback );
+ $mesg->body( $body );            # replace message body
+ $body = $mesg->body();           # get message body
+ $head = $mesg->header();         # get message header
 
- $mesg->routeTo( $mode => $session, $event, @eventArgs );
-
- $resp = $MsgClass->new( $mesg, "back atcha!" );
- $resp->reply();                      # activate any "auto-reply"
-
- $resp->reply( @additionalArgs );     # "$resp" included w/reply
+ print $mesg->dump();             # useful when debugging
+ print $mesg->header()->dump();   # useful when debugging
 
 
- # Note/FIX: "addRouteBack" does not default mode to "setMode" val
+ # Accessing Header Components
 
- # Sending and Receiving Messages      (Outside POE environment)
+ $mesgID = $mesg->id();           # unique message identifier
+ $respID = $mesg->r2id();         # 'In Response To' identifier
 
- $MsgClass->write( $fh, $message );   # write fh (sock, whatever)
- $mesg->write( $fh );                 # write fh (sock, whatever)
+ $bool   = $mesg->hasRouting();
+ $bool   = $mesg->hasRouteTo();
+ $bool   = $mesg->hasRouteBack();
 
- $resp = $MsgClass->read( $fh );      # read fh (sock, whatever)
- $resp = $mesg->read( $fh );          # read fh (sock, whatever)
+ $routing= $mesg->nextRouteType();      # post, call, remote or ''
+ $bool   = $mesg->nextRouteIsRemote();  # 1 if remote    or 0 if not
+ $bool   = $mesg->nextRouteIsLocal();   # 1 if post|call or 0 
+ $bool   = $mesg->nextRouteIsPost();    # 1 if post      or 0 
+ $bool   = $mesg->nextRouteIsCall();    # 1 if call      or 0 
+
+
+ # Set default 'mode' used with 'route / routeto / routeback'
+
+ $mesg->setMode( "call" );        # original default: "post"
+
+ $mode = $mesg->mode();           # one of 'post' or 'call'
+
+
+ # Routing Messages Within the POE environment
+ # note that '$session' defaults to current session context
+ # (and the first parameter ('mode') defaults to 'post'
+ # unless the 'setMode()' method was used to change this)
+
+ $mesg->addRouteTo( "post", $session, $to_event, @stateArgs );
+
+ $mesg->addRouteBack( "post", "", 'my_handler', @stateArgs );
+
+ $resp = $mesg->route( @routeArgs );
+
+
+ # Routing Messages Outside the POE environment
+ # 'sync' expects some sort of response message 
+ # 'asynch' expects only 'delivery status' response
+
+ $mesg->addRemoteRouteTo( $host, $port "sync" );
+
+ $mesg->addRemoteRouteTo( $host, $port "async" );
+
+ $response = $mesg->route( @routeArgs );
+
+
+ # Processing Messages Within the POE environment
+ # (brief example of 'object_state' event method)
+
+ sub handle_send_response
+ {   my($self, $stateArgs, $routeArgs) = @_[ OBJECT, ARG0, ARG1 ];
+
+     my $message  = $routeArgs->[0];
+     my $response = $MsgClass->new( $message, "" );
+
+     return $response->route();
+ }
+
+
+ # Directly Sending and Receiving Messages Outside the POE environment
+ # where '$fh' is a file handle, socket or whatever
+
+ $MsgClass->write( $fh, $mesg );
+ $mesg->write( $fh );
+
+ $response = $MsgClass->read( $fh );
+ $response = $mesg->read( $fh );
 
 
 =head1 DESCRIPTION
 
 This class is a starting point for creating a generic application
 messaging protocol. The intent is for this to be used as a foundation
-when building network client/server applications, Web services, etc.
+when building network and/or event-driven applications such as Web 
+services.
 
 Features of POE::Event::Message include the following.
 
@@ -627,9 +701,22 @@ that work both inside and outside POE-based applications.
 
 =item *
 
-The messages are designed to be plug-compatible with 
-POE's existing 'postback' mechanism. I refer to this as 
-a 'routeback' mechanism instead. 
+A 'local' message routing mechanism is designed to be 
+plug-compatible with POE's existing 'postback' mechanism. 
+This is referred to as a 'routeback' mechanism in this class. 
+
+=item *
+
+A 'remote' message routing mechanism can be used by message-based
+client scripts, or perhaps as a simple alternative to the 'IKC' 
+(Inter Kernel Communication) mechanism.
+
+=item *
+
+Messages are delivered based on the type of the destination. 
+A 'local' routing will trigger a POE 'post' or 'call' event, 
+while a 'remote' routing invokes a socket connection to a 
+particular host and port. 
 
 =item *
 
@@ -642,36 +729,100 @@ knowing or caring.
 
 =item *
 
-Messages are delivered based on the type of the destination. 
-A 'local' routing will trigger a POE 'post' or 'call' event, 
-while a 'remote' routing invokes a socket connection to a 
-particular host and port. 
-
-=item *
-
 Messages can also be sent through a file handle, such as 
 from a non-POE child to a POE-based parent process.
 
+=item *
+
+Messages can contain almost anything including binary data
+and complex Perl data structures.
+
 =back
 
+Messages are created using the 'L<new()|"new">' method or the 
+L<envelope()|"envelope">' or L<package()|"package">' methods.
+A message body can be add during creation or afterward using
+the L<body()|"body">' method.
+
+Responses to messages can be created using the same message creation
+methods but with the originating message as an added argument.
+This way, the unique message identifier on the originating message
+is added to the reply as an 'in reply to' identifier. This allows
+correlating originating messages with their replies, if necessary.
+
+Message routing is flexible and can be either 'B<local>' or 'B<remote>'.
+The 'L<addRouteTo()|"addRouteTo">' and 'L<addRouteBack()|"addRouteBack">' 
+methods are used to add 'B<local>' routing to POE-based event services.
+This type of routing triggers POE's internal 'post' or 'call' mechanism
+for asynchronous or synchronous message delivery.
+
+The 'L<addRemoteRouteTo()|"addRemoteRouteTo">' and 
+'L<addRemoteRouteBack()|"addRemoteRouteBack">' methods are used to
+add 'B<remote>' routing of messages to services on remote hosts.
+This type of routing triggers 'L<read()|"read">' and 'L<write()|"write">'
+methods to send and receive messages across network sockets. Remote
+delivery can be either synchronous (expecting an immediate response 
+message) or asynchronous (where response is only a 'delivery status' of 
+either successful or unsuccessful).
+
+Multiple forward and/or reverse routing destinations can be added to a 
+given message. Additional local and/or remote routing destinations can 
+be added at any time during processing. This is useful, for example, to 
+interrupt the original message flow, as none of the original participants 
+need be aware of any interruption(s).
+
+When adding a routing destination, a list of 'B<StateArgs>' can be 
+included. These will be available to the receiving event method when the 
+message is delivered (just like POE's postback/callback StateArgs list).
+
+The 'L<route()|"route">' method is used to actually route messages
+to their destinations. When routing messages, a list of 'B<RouteArgs>' 
+can be included. These will be available to the receiving event method 
+when the message is delivered (just like POE's postback/callback 
+AdditionalArgs list). I<Note that the routed message object will be
+delivered as the first element of the 'B<RouteArgs>' list reference>.
+
+This class does not explicitly use POE. This allows
+'light weight' scripts that run outside the full POE environment to
+use this messaging protocol. When POE is not used, only the 'B<remote>'
+mechanism is available. This is because the 'post' and 'call' routing 
+semantics rely on the POE kernel. 
+
+Note also that 'B<local>' routing destinations using 'post' and 'call' can
+be created outside of POE. If, for example, messages are sent via a 'remote 
+routing' to a POE-based process, that process can then 'L<route()|"route">' 
+messages using 'post' and/or 'call' to the specified event service 
+destination(s).
+
+When running outside of the POE environment either explicitly add a 
+'remote' routing -or- use the L<routeIsRemote()|"routeIsRemote"> method 
+(as shown in the L<Synopsis|"SYNOPSIS"> section, above) to determine if 
+the L<route()|"route"> method can be called on a given message object.
 
 =head2 Constructor
 
 =over 4
 
+=item new ( [ Message ] [, Body ] )
+
 =item new ( [ Header ] [, Body ] )
 
-This method instantiates a new message object. Optionally it
-can be used to create a response to an existing message. A
-message body can also be specified during creation.
+This method instantiates a new message object. Optionally it can be 
+used to create a response to an existing message. A message body can 
+also be specified during creation and/or added later.
 
 =over 4
 
-=item Header
+=item Message | Header
 
-The optional B<Header> argument, when included, is expected
-to be an original message of this class. This mechanism is
-used to create a 'response' to the original message.
+The optional B<Message> or B<Header> argument, used when creating a 
+response to an originating message, is expected to be either the 
+originating message or the header object from the originating message. 
+
+Every message has a guaranteed unique message identifier. The response
+mechanism is used to include the original message identifier as an
+'in response to' identifier. This 'r2id' can then be used, when necessary,
+to correlate original messages with any responses received.
 
 =item Body
 
@@ -711,9 +862,9 @@ structure as explained in the L<new> method, above.
 =back
 
 
-=item call ( Service, Event [, Args ] )
+=item call ( [ Session ], Event [, Args ] )
 
-=item post ( Service, Event [, Args ] )
+=item post ( [ Session ], Event [, Args ] )
 
 These methods provide direct message routing within a POE application 
 and will invoke the corresponding method on the POE Kernel for immediate,
@@ -722,9 +873,10 @@ dispatch (post).
 
 =over 4
 
-=item Service
+=item Session
 
-This argument is the name of an active POE 'B<Service>'.
+This optional argument is the name of an active POE 'B<session>'.
+When omitted, the current POE 'session context' is used.
 
 =item Event
 
@@ -738,15 +890,15 @@ with either of the POE 'post' or 'call' event generators.
 =back
 
 
-=item addRouteTo ( Mode, Service, Event [, Args ] )
+=item addRouteTo ( Mode, [ Session ], Event [, StateArgs ] )
 
-=item addRouteBack ( Mode, Service, Event [, Args ] )
+=item addRouteBack ( Mode, [ Session ], Event [, StateArgs ] )
 
 =over 4
 
-These methods add B<auto-routing> capabilities to messages of this
+These methods add B<local> capabilities to messages of this
 class. These add routing entries to the message header that can
-then be invoked via the L<route> methods, explained next.
+then be invoked via the L<route> methods, explained below.
 
 =item Mode
 
@@ -756,27 +908,63 @@ either the corresponding method on the POE Kernel for immediate,
 synchronous event dispatch (call) or queued for asynchronous event 
 dispatch (post).
 
-=item Service
+=item Session
 
-This argument is the name of an active POE 'B<Service>'.
+This optional argument is the name of an active POE 'B<session>'.
+When omitted, the current POE 'session context' is used.
 
 =item Event
 
-This argument is the name of an active POE 'B<Service>'.
+This argument is the name of an 'B<event>' in an active 
+POE 'Session'.
 
-=item Args
+=item StateArgs
 
-The B<Args> parameter is an optional list that can be included
+The B<StateArgs> parameter is an optional list that can be included
 with either of the POE 'post' or 'call' event generators.
 
 =back
 
 
-=item route ( [ Args ] )
+=item addRemoteRouteTo ( Host, Port, Mode, [ Session ], Event [, StateArgs ] )
 
-=item routeto ( [ Args ] )
+=item addRemoteRouteBack ( Host, Port, Mode, [ Session ], Event [, StateArgs ] )
 
-=item routeback ( [ Args ] )
+=over 4
+
+These methods add B<remote> capabilities to messages of this
+class. These add routing entries to the message header that can
+then be invoked via the L<route> methods, explained below.
+
+=item Host
+
+The remote host to which the message will be routed.
+
+=item Port
+
+The network port on the remote host.
+
+=item Mode
+
+The B<Mode> must be a string equal to one of 'B<sync>' or 'B<asynch>'.
+Later, when one of the L<route> methods is used, will then either
+expect an immediate response message ('sync') or simply return a
+status indicating success or failure of 'B<message delivery>' only,
+and not wait for any other response. This is useful when some other 
+process, perhaps on another host, will handle the response, if any.
+
+The remaining arguments are identical as the corresponding
+L<addRouteTo()|"addRouteTo"> and L<addRouteBack()|"addRouteBack"> 
+methods described above.
+
+=back
+
+
+=item route ( [ RouteArgs ] )
+
+=item routeto ( [ RouteArgs ] )
+
+=item routeback ( [ RouteArgs ] )
 
 These methods add B<auto-routing> capabilities to messages of this
 class. These extract routing entries from the message header that are
@@ -785,49 +973,58 @@ routing entry was created with a B<Mode> of 'B<call>') or queued for
 delayed dispatch (when the routing entry was created with a B<Mode> 
 of 'B<post>').
 
+The B<route> method will first extract B<routeTo> entries and, when
+empty, will extract B<routeBack> entries.
+
 The B<routeto> method will extract only those entries created using the 
 B<L<addRouteTo>> method, shown above, while the B<routeback> method will 
 extract only those entries created using the B<L<addRouteBack>> method,
 also shown above.
 
-The B<route> method will first extract B<routeTo> entries and, when
-empty, will extract B<routeBack> entries.
+If/when the routing list(s) are empty, these methods will silently
+do nothing. If this should cause an error, use the B<hasRouting>,
+B<hasRouteTo> or B<hasRouteBack> methods to determine when an 
+error should be generated.
 
-NOTE:
-These methods are designed to be plug-compatible with POE's 
-'B<postback>' mechanism. These methods add the ability to introduce 
-multiple forward and/or reverse event destinations and can be used
-to temporarially interrupt normal message routing (and none of the
+NOTE: The 'B<local>' routing mechanism is designed to be plug-compatible 
+with POE's existing 'B<postback>' mechanism. These methods add the ability 
+to introduce multiple forward and/or reverse event destinations and can be 
+used to temporarially interrupt normal message routing (and none of the
 handlers that expect to process the message will even be aware of 
 the interruption).
 
+NOTE: The 'B<remote>' routing mechanism can be used as a simple alternative
+to the 'IKC' (Inter Kernel Communication) mechanism.
+
 =over 4
 
-=item Args
+=item RouteArgs
 
-The B<Args> parameter is an optional list that can be added
-when any of the 'route' methods are invoked. These arguments
+The B<RouteArgs> parameter is an optional list that can be added
+when any of the 'route()' methods are invoked. These arguments
 are then received by whichever event handler happens to
-receive the routed message. Note that the message itself is
-already included as the first argument in the list.
+receive the routed message. I<Note that the message itself is
+included as the first argument in the B<RouteArgs> list>. 
+See the L<Synopsis|"Synopsis"> section, above, and the
+L<Examples|"Examples"> section, below, for usage syntax.
 
 =back
 
 
-=item send ( FH, Message )
+=item send ( FH [, Message ] )
 
-=item write ( FH, Message )
+=item write ( FH [, Message ] )
 
 These methods provide the ability to send the message object
-through an open file handle.
+through an open file handle. The 'B<FH>' parameter should be
+an open file handle and the 'B<Message>' parameter, when used,
+is expected to be an object of this class. When the 'B<Message>'
+is omitted, the current object is sent through the file handle.
 
 WARN: Use caution when using these methods with messages containing 
 Perl 'CODE references.' While the underlying 'Storable' module does
 now support passing CODE refs, this has not been tested here.
 
-TODO: The B<Message> parameter should be optional and, when
-omitted, send the current object through the file handle.
-
 =over 4
 
 =item FH
@@ -836,7 +1033,11 @@ An open file handle.
 
 =item Message
 
-A message object of this class.
+A message object of this class. Optional when these methods are
+called as object methods on an existing object, and required when 
+these methods are called as class methods.
+
+When omitted, B<Message> defaults to the current message object.
 
 =back
 
@@ -862,6 +1063,55 @@ An open file handle.
 =back
 
 
+=item id ()
+
+This method is used to determine the unique message identifier 
+of the current message object.
+
+
+=item inResponseToId ()
+
+=item r2id ()
+
+For 'response messages' these methods are used to determine the 
+message identifier of the original message.
+
+Note: This is why, when creating a response message, it is important to
+include an originating message as the first argument to the 'L<new()|"new">' 
+method. This way the original message ID can be collected and included 
+as the 'B<r2id>'.
+
+
+=item hasRouting ()
+
+=item hasRouteTo ()
+
+=item hasRouteBack ()
+
+Miscellaneous methods used to determine 'header' state. Each of these
+returns the next 'routing' header entry, if any, or an 'undef' value.
+The 'routing' entry returned is retained in the message header.
+
+
+=item nextRouteType ()
+
+=item nextRouteIsRemote ()
+
+=item nextRouteIsLocal ()
+
+=item nextRouteIsPost ()
+
+=item nextRouteIsCall ()
+
+Miscellaneous methods used to determine 'header' state. Each of these,
+with the exception of 'B<nextRouteType()'>, returns a boolean value
+where 'B<0>' indicates the test is false and 'B<1>' indicates the test
+is true.
+
+The 'B<nextRouteType()'> method returns a string value that is one of
+'B<remote>', 'B<call>', 'B<post>' or '' (an empty string).
+
+
 =item status
 
 This method returns the current error status of a message object.
@@ -877,6 +1127,128 @@ and the message header object, when one exists.
 
 =back
 
+=head1 EXAMPLES
+
+The following is an incomplete example of creating a network interface 
+that accepts message objects from a client script and returns message 
+responses. 
+
+For a complete working example of this see the 'TCP' and 'Cmd' server 
+classes in the 'B<POE-Component-GCS>' distribution available on CPAN.
+
+ package POE::Component::GCS::Server::TCP;
+
+ $ServerAlias = "MsgServer";
+ $ServerPort  = 12345;
+
+ sub spawn
+ {   my($self) = @_;
+
+     POE::Component::Server::TCP->new(
+         Port     => $ServerPort,    # Bind Port
+         Alias    => $ServerAlias,   # necessary for 'routeback' response
+
+         ClientInput   => \&handle_msg_input,
+         ClientFilter  => "POE::Filter::Reference",
+
+         # Most of the events we need are pre-defined by POE.
+         # Add one here to so we can create a "routeback"
+         # event target to simplify the response mechanism.
+
+         InlineStates => {
+              client_msg_output => \&handle_msg_output,
+         },
+     );
+     return;
+ } 
+
+ sub handle_msg_input
+ {   my($kernel, $heap, $session, $input) = @_[KERNEL, HEAP, SESSION, ARG0];
+
+     # The $input variable here is expected to be a "$MsgClass" object.
+     # Create a msg routeback wormhole using "post" for asynch delivery.
+
+     $input->addRouteBack( post => undef, "client_msg_output", undef );
+
+     # Then "dispatch" the message to some command processor
+
+     $CmdClass->dispatch( $input );        # (non-POE method)
+     return;
+ }
+
+ sub handle_msg_output                # 'client_msg_output' event handler
+ {   my($kernel, $heap, $session, $state_args, $result_args) =
+     @_[ KERNEL,  HEAP,  SESSION,  ARG0,        ARG1       ];
+
+     # This handler is the "routeback" target that returns data
+     # to client ($reply is expected to be a "$MsgClass" object).
+     # Here we simply return it to the waiting Msg-based client.
+
+     my $reply = $result_args->[0];
+
+     # The "client" attribute was added by POE to the heap to
+     # facilitate sending the correct response to the correct
+     # client. All we need to do is "put" the "$reply".
+
+     $heap->{client}->put( $reply );
+
+     my $sessionId = $session->ID;
+     $Log->write(4, "$MsgAlias: client reply sent (cid=$sessionId)");
+     return;
+ }
+
+Then, to continue this example for a moment more, if the '$CmdClass'
+containing the 'dispatch()' method were to determine that the incoming
+message was a valid message object but an illegal commnad, the entire
+response mechanism is as simple as this next method. In this case,
+the '$reply' sent ('put') just above contains the following error.
+
+ package POE::Component::GCS::Server::Cmd;
+
+ sub _invalid_request
+ {   my($class, $message, $error_text) = @_;
+
+     $error_text ||= "invalid command";
+
+     # We must reply with something or TCP clients will hang. 
+     # Simply closing the socket results in a "no response 
+     # from server" error on the client side.
+
+     my $command  = $message->body();
+     my $response = $MsgClass->new( $message, $command );
+     $response->setErr( -1, $error_text );
+    
+     $response->route();       # return error to waiting TCP client
+     return;
+ }
+
+
+=head1 WARNINGS
+
+When routing message objects through file handles or sockets use caution 
+with messages containing Perl 'CODE references.' While the underlying 
+'Storable' module does now support passing CODE refs, this has not been 
+tested here.
+
+Also note that this class does not explicitly use POE. This allows
+'light weight' scripts that run outside the full POE environment to
+use this messaging protocol. When POE is not used, the 'route()'
+mechanism is B<only> available when the next routing is to a remote
+host. This is because the 'post' and 'call' routing semantics rely
+on the POE kernel. Without POE, only the 'read' and 'write'
+mechanism is available to send messages across file handles or sockets.
+
+However, the 'addRouteTo()' and 'addRouteBack()' methods B<are> 
+available outside of POE, and can be used to add 'post' and 'call'
+routing. For example, if messages are sent via a 'remote route' to 
+a POE-based process, that process can then 'route()' messages which
+contain pre-defined 'post' and/or 'call' destinations.
+
+When running outside of the POE environment either explicitly add a 
+'remote' routing -or- use the B<routeIsRemote()> method (as shown in the 
+L<Synopsis|"SYNOPSIS"> section, above) to determine if the B<route()> 
+method can be called on a given message object.
+
 =head1 DEPENDENCIES
 
 This class depends upon the following classes:
@@ -886,11 +1258,7 @@ This class depends upon the following classes:
  POE::Driver::SysRW
  POE::Filter::Reference
 
- POE::Kernel (optional, but required for some methods)
-
-=head1 INHERITANCE
-
-None currently.
+ POE::Kernel (optional, but required for 'post' and 'call' message routing)
 
 =head1 SEE ALSO
 
@@ -898,9 +1266,12 @@ See
 L<POE::Event::Message::Header> and
 L<POE::Event::Message::UniqueID>.
 
+See the POE-Component-GCS distribution available on CPAN for a complete 
+working model of a Generic Client/Server that uses message-based events.
+
 =head1 AUTHOR
 
-Chris Cobb, E<lt>nospamplease@ccobb.netE<gt>
+Chris Cobb [no dot spam at ccobb dot net]
 
 =head1 COPYRIGHT
 
